function [Quality, Edistortion, Adistortion, Rdistortion, separation, ...
	 Rdistortion1, invScatter, SOLUTIONS] = ...
         new_kmeans(data, c, label, repeats)  

% ----------------------------------------------------------------------------%
% this method is applied in Euclidean space.                                  %
% The similarity between pair of points is in terms of Euclidean distance.    %
%                                                                             %
% For correlated data (or data whose length varies a lot but whose similarity %
% lies in direction), it is advised to normalized the data to unit length     %
% before apply to this kmeans procedure.                                      %
%                                                                             %
% The input is a data matrix, where each row is a point                       %
%                                                                             %
%                                                                             %
% This program return a variaty of quality measures of the clustering         %
% (in the () is the perfect result one could get)                             %
% 1. Qualityk: the purity measure with respect to the known labels. (1)       %
% 2. distortion: average distance of each point to its cluster center. (0)    %
% 3. distortion1: same as distortion, but using angle rather then distance (1)%
%    normalised.  (0)                                                         %
% 4. dis1: Homogeneity/Centroid Seperation. ratio between distortion          %
%    and average distance of each center to the total mean. (0)               %
% 5. dis2: Homogeneity/Heterogeneity as of the Click paper. ratio between     %
%    distortion and average pairwise distances of points not in the same      %
%    cluster.     (0)                                                         %
% 6. sc: (1)                                                                  %
%                                                                             %
%    Eric Xing, UC Berkeley, 12/20/2001                                       %
% ----------------------------------------------------------------------------%

s=size(data);
M=s(2);                   % dimensional of data
N=s(1);                   % number of data points 
original_label= label;

x2=data';                 % original data, with each column being a sample

% ***************************************************************

original=original_label;
for r = 1:repeats
  [solution, u] = Kmeans(x2, c);

  Quality(r) = purity(solution, original);
  Edistortion(r) = Distortion(x2, u, solution);
  Adistortion(r) = A_Distortion(x2, u, solution);
  Rdistortion(r) = R_Distortion(x2, u, solution);
  separation(r) = Separation(x2, u, solution);
  Rdistortion1(r) = Edistortion(r)/separation(r);
  invScatter(r) = scattering(x2, u, solution);
  SOLUTIONS(r,:) = solution;

end




% --------------------------------------------------
function d = distance(a1, a2)
    d = norm(a1-a2);
%    d = exp(1-(1-dot(x2(:,n), x2(:,i))/M)/sigma);  correlation
%    temp = norm(a1)*norm(a2);
%    if temp ~= 0 
%      d = 1-a1'*a2/temp; % cosine;
%    else 
%      d = 1;
%    end
      

% -------------------------------------------------    
function Q = purity1 (q, q1)
K = length(q);
ccc=0;
for i=1:K
  for j=i+1:K
    ccc = ccc + ((q(i)==q(j))==(q1(i)==q1(j)));
  end
end
Q = ccc/(0.5*K*(K-1));


% ------------- calculate Euclidean-distortion ---------------------
function distortion = Distortion(x2, u, solution)

  [d, N] = size(x2);
  [d, c] = size(u);
 
  distortion = 0;
  for i=1:N
    distortion = distortion + distance(x2(:,i),u(:,solution(i)));
  end
  distortion = distortion/N;


% -------------- calculate angular distortion --------------
function distortion = A_Distortion(x2, u, solution)

  [d, N] = size(x2);
  [d, c] = size(u);
 
  for k=1:c
    u1(:,k) = u(:,k)/norm(u(:,k));
  end

  distortion = 0;
  for i=1:N
    temp = norm(x2(:,i));
    if temp~=0
       distortion = distortion + x2(:,i)'*u1(:,solution(i))/norm(x2(:,i));
    else 
       distortion = distortion;
    end
  end
  distortion =  distortion/N;


% ---- relative Euclidean-distortion: distortion/clusterMean-separation ----
function distort = R_Distortion(x2, u, solution)

  [d, N] = size(x2);
  [d, c] = size(u);
 
  dis = 0;
  for i=1:N
     dis = dis + distance(x2(:,i),u(:,solution(i)));
  end
  dis = dis/N;
  
  dis1 = 0;
  total_mean = mean(x2,2);
  for i=1:c
     dis1 = dis1 + distance(u(:,i),total_mean);
  end
  dis1 = dis1/c;

  distort = dis/dis1;


% ---- Euclidean-point-separation --------------------
function dist1 = Separation(x2, u, solution)

  [d, N] = size(x2);
  [d, c] = size(u);
  
  dis1 = 0;
  count = 0;
  for i = 1:N
  for j = i+1:N
      if (solution(i)~=solution(j))
         dis1 = dis1 + distance(x2(:,i),x2(:,j));
         count = count + 1;
      end
  end
  end
  dist1 = dis1/count;



% ---- relative distortion: clusterMean-cov/inner-cluster-sample-cov 
function smeasure = scattering(x, u, solution)
     [d, N] = size(x);
     [d, c] = size(u);
     total_mean = mean(u,2);

     Pb = zeros(d,d);
     P =  zeros(d,d);
     for k = 1:c
       Pb = Pb + (u(:,k)-total_mean)*(u(:,k)-total_mean)';
     end
   
     for i=1:N
       P = P + (x(:,i)-u(:,solution(i)))*(x(:,i)-u(:,solution(i)))';
     end

     smeasure = trace(inv(P+0.0001*eye(d,d))*Pb);









% ************** k-means with constraints ***************
function [solution, u] = Kmeans(x2, c)

[M,N]=size(x2);

seed=zeros(1,c)-1;
for i=1:c
 tempseed = round(rand*(N-1)+1);
 while ~isempty(find(seed==tempseed))
   tempseed = round(rand*(N-1)+1);
 end
 seed(i)=tempseed;           % non-overlapping random centroid of each cluster
 u(:,i)=x2(:,seed(i));       % initialize all cluster centroid
end

dist=zeros(c,N);
dd=zeros(1,N);        % distance to the closest neighbor
q=zeros(1,N);         % labeling trace
qtemp=zeros(1,N);     % temp labeling trace
tempu=zeros(M,c);     % cumulative center for each class  
z=zeros(1,c);         % counts for each cluster

t=1;
compare=zeros(1,N);

while sum(compare)~=N
  
  qtemp=q;           % lables from last iteration
  
  % K-mean for each point x2(n):_________
  for n=1:N
 
    for i=1:c
      dist(i,n)=distance(x2(:,n),u(:,i));
    end
    [dd(n),q(n)]=min(dist(:,n));        % max-correlation
    
    k=q(n);
    tempu(:,k)=tempu(:,k)+x2(:,n); % cumulative center for each class
    z(k)=z(k)+1;                   % counts for each class

  end

  newseed_record = zeros(1,c)-1;
  for i=1:c
    if z(i)==0

      tempseed = round(rand*(N-1)+1);
      while ~isempty(find(newseed_record==tempseed))
        tempseed = round(rand*(N-1)+1);
      end
      newseed_record(i) = tempseed;
      u(:,i)=x2(:,tempseed);

    else
      u(:,i)=tempu(:,i)/z(i);       % new center for each class
    end
  end


  tempu=zeros(M,c);
  z=zeros(1,c);
  
  compare=(qtemp==q);
  t=t+1;
  if t>200
   compare=N;
  end
end

kmeans_label=q;
solution=kmeans_label;
